import sys
import cmd
import bdb
import pdb
import threading
import time
import JESDBVariableWatcher


line_prefix = '\n-> '
prompt = '(jesdb) '

SPEED_FACTOR = 20.0
class JESDebugger(pdb.Pdb):
    
    
    def __init__(self, interpreter):
        pdb.Pdb.__init__(self)
        self.speed_factor = SPEED_FACTOR
        self.MAX_SPEED = int(3 * SPEED_FACTOR)
        self.text_mode = 0  # command line interface if on
        self.running = 0    # running is when we are inside a debugger run, i.e. we
                            # we're debugging a command(not waiting for a new command)
        self.lock = threading.Lock()    # we need a lock for the wait mechanism
        self.cond = threading.Condition(self.lock)
        self.interpreter = interpreter
        self.speed = int(2 * SPEED_FACTOR)      # cycles per second

        self.controlPanel = JESDBVariableWatcher.DBControlPanel(self)
        self.watcher = JESDBVariableWatcher.JESDBVariableWatcher(self, self.controlPanel)  # not necessarily have to be showing
        self.last_time = -1;
        
        self.cmd = None
        self.reset()

    def setSpeed(self, speed):
        self.speed = speed
        # notify control panel
        self.controlPanel.refreshSlider()


    def addVariable(self, var):
        self.watcher.addVariable(var)

    def removeVariable(self, var):
        self.watcher.removeVariable(var)
        
    def endExecution(self):
        self.watcher.endExecution()
        self.controlPanel.stop() # this sets the button states
	
    # Override Bdb methods (except user_call, for now)
    def user_line(self, frame):
        """This function is called when we stop or break at this line."""
        #print 'in user_line---'
        if frame.f_code.co_filename == self.interpreter.program.filename:
            #if self.break_here(frame) or not self.running:
            self.running = 1
            self.interaction(frame, None)
        

    def user_return(self, frame, return_value):
        pass

    def user_exception(self, frame, (exc_type, exc_value, exc_traceback)):
        pass


    def stopThread(self):
        self.running = 0
        self.step()        
        
    # this is the main interface from the outside
    def runCommand(self, cmd):
        self.cmd = cmd
        self.lock.acquire()
        self.cond.notifyAll()
        self.lock.release()
        
    def step(self):
        self.lock.acquire()
        self.cond.notifyAll()
        self.lock.release()
        
    def print_frame(self, frame):
        self.print_stack_entry((frame, frame.f_lineno))

        
    # General interaction function    
    def interaction(self, frame, traceback):
        
        if self.text_mode:
            cmd = ''
            stop = 0
            self.print_frame(frame)
            while not stop:
                self.setup(frame, traceback)
                self.interpreter.jesThread.threadCleanup()
                self.lock.acquire()
                self.cond.wait()     # wait for the program to signal
                self.lock.release()
                self.interpreter.jesThread.initialize()
                stop = self.onecmd(self.cmd)
                
        else:
            # if we're not in stop mode then we're in...
            # unique to JESDB...the running debug mode!!
            
            # get line and frame number
            import linecache, time
            self.setup(frame, traceback)
            lineno = frame.f_lineno
            filename = self.interpreter.program.filename
            line = linecache.getline(filename, lineno)
            self.watcher.snapShot(lineno, line)
            if self.speed < self.MAX_SPEED:
                if self.speed == 0:
                    self.lock.acquire()
                    self.cond.wait()
                    self.lock.release()
                else:
                    t = time.time()
                    period = SPEED_FACTOR / self.speed
                    if self.last_time == -1:
                        pause = period
                    else:
                        pause = period - (t - self.last_time)
                    self.last_time = t
                    if pause > 0:
                        time.sleep(pause)
            if not self.running:
                self.interpreter.jesThread.stop() # stop myself
                
            
                
    
    def run(self, cmd, globals=None, locals=None):
        self.running = 0
        self.watcher.clear()
        self.controlPanel.run()
        self.last_time = time.time()
        pdb.Pdb.run(self, cmd, globals, locals)
        self.running = 0
        
        
        
    def runeval(self, cmd, globals=None, locals=None):
        self.running = 0
        self.watcher.clear()
        self.controlPanel.run()
        self.last_time = time.time()
        pdb.Pdb.runeval(self, cmd, globals, locals)
        self.running = 0

    
