# 
# Media Wrappers for "Introduction to Media Computation"
# Started: Mark Guzdial, 2 July 2002
# Revisions:
# 18 June 2003: (ellie)
#    Added (blocking)play(AtRate)InRange methods to Sound class 
#      and global sound functions
#    Changed getSampleValue, setSampleValue to give the appearance 
#      of 1-based indexing
#    Added getLeftSampleValue and getRightSampleValue to Sound class
# 14 May 2003: Fixed discrepancy between getMediaPath and setMediaFolder (AdamW) 
# 8 Nov: Fixed getSamplingRate (MarkG)
# 1 Nov: Fixed printing pixel
# 31 Oct: Fixed pickAColor (MarkG)
# 30 Oct: Added raises, fixed error messages. Added repaint (MarkG)
# 10 Oct: Coerced value to integer in Sound.setSampleValue   (MarkR)
# 2 Oct:  Corrected calls in setSampleValueAt  (MarkG): 
# 30 Aug: Made commands more consistent and add type-checking (MarkG)
# 2 Aug: Changed to getSampleValueAt and setSampleValueAt (MarkG)
# 1 Dec 2005: made max makeEmptySound size 600
#             made max makeEmptyPicture dimensions 10000x10000
#             fixed the off-by-one error in makeEmptyPicture 

# TODO:
## Fix HSV/RGB conversions -- getting a divide by zero error when max=min

# Get the Java Pieces
# import JavaSound
# import JavaPicture
# import JavaPixel
import JavaMusic

###################################
# integrating in new java classes #
###################################

import java.awt.Font
import java.awt as awt
import javax.swing as swing
import java.util
import sys
import os
import math
import traceback
import user
import Picture
import Pixel
import Sound
import FrameSequencerTool
import SimpleInput
import SimpleOutput
import Sample
import Samples

import org.python.core.PyString as String

# Support a media shortcut

#mediaFolder = os.getcwd() + os.sep
mediaFolder = user.home + os.sep

# Store last pickAFile() opening

_lastFilePath = ""

#Set a path to the media for those who want to have a shortcut (5/14/03 AW)
def setMediaPath():
    global mediaFolder
    file = pickAFolder();
    mediaFolder = file+os.sep
    print "New media folder: "+mediaFolder

def getMediaPath(filename):
    global mediaFolder
    file = mediaFolder+filename
    if not os.path.isfile(file):
        print "Note: There is no file at "+file
    return file


#And for those who think of things as folders (5/14/03 AW)  
def setMediaFolder():
    global mediaFolder
    file = pickAFolder();
    mediaFolder = file+os.sep
    print "New media folder: "+mediaFolder

def setTestMediaFolder():
    global mediaFolder
    mediaFolder = os.getcwd() + os.sep

def getMediaFolder(filename):
    global mediaFolder
    file = mediaFolder+filename
    if not os.path.isfile(file) or not os.path.isdir(file):
        print "Note: There is no file at "+file
    return file
    
def getShortPath(filename):
    dirs = filename.split(os.sep)
    if len(dirs) < 1:
        return "."
    elif len(dirs) == 1:
        return dirs[0]
    else:
        return (dirs[len(dirs) - 2] + os.sep + dirs[len(dirs) - 1])

##
## Global sound functions
##
def makeSound(filename):
    global mediaFolder
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not os.path.isfile(filename):
        print "There is no file at "+filename
        raise ValueError 
    return Sound(filename)

# MMO (1 Dec 2005): capped size of sound to 600
def makeEmptySound(size):
    if size > 600:
        print "makeEmptySound(size): size must be 600 seconds or less"
        raise ValueError
    return Sound(size * Sound.SAMPLE_RATE)

def getSamples(sound):
    if not sound.__class__ == Sound:
        print "getSamples(sound): Input is not a sound."
        raise ValueError 
#    return Samples(sound)
    return Samples.getSamples(sound)

def play(sound):
    if not sound.__class__ == Sound:
        print "play(sound): Input is not a sound."
        raise ValueError 
    sound.play()

def blockingPlay(sound):
    if not sound.__class__ == Sound:
        print "blockingPlay(sound): Input is not a sound."
        raise ValueError 
    sound.blockingPlay()

def playAtRate(sound,rate):
    if not sound.__class__ == Sound:
        print "playAtRate(sound,rate): Input is not a sound."
        raise ValueError 
    # sound.playAtRate(rate)
    sound.playAtRateDur(rate,sound.getLength())

def playAtRateDur(sound,rate,dur):
    if not sound.__class__ == Sound:
        print "playAtRateDur(sound,rate,dur): Input is not a sound."
        raise ValueError 
    sound.playAtRateDur(rate,dur)

#20June03 new functionality in JavaSound (ellie)
def playInRange(sound,start,stop):
        if not sound.__class__ == Sound:
                print "playInRange(sound,start,stop):  Input is not a sound."
                raise ValueError
        # sound.playInRange(start,stop)
        sound.playAtRateInRange(1,start-1,stop-1)
        
#20June03 new functionality in JavaSound (ellie)
def blockingPlayInRange(sound,start,stop):
        if not sound.__class__ == Sound:
                print "blockingPlayInRange(sound,start,stop): Input is not a sound."
                raise ValueError
        # sound.blockingPlayInRange(start,stop)
        sound.blockingPlayAtRateInRange(1,start-1,stop-1)

#20June03 new functionality in JavaSound (ellie)
def playAtRateInRange(sound,rate,start,stop):
        if not sound.__class__ == Sound:
                print "playAtRateInRAnge(sound,rate,start,stop): Input is not a sound."
                raise ValueError
        sound.playAtRateInRange(rate,start - 1,stop - 1)

#20June03 new functionality in JavaSound (ellie)
def blockingPlayAtRateInRange(sound,rate,start,stop):
        if not sound.__class__ == Sound:
                print "blockingPlayAtRateInRange(sound,rate,start,stop):  Input is not a sound."
                raise ValueError
        sound.blockingPlayAtRateInRange(rate, start - 1,stop - 1)

def getSamplingRate(sound):
    if not sound.__class__ == Sound:
        print "getSamplingRate(sound): Input is not a sound."
        raise ValueError 
    return sound.getSamplingRate()

def setSampleValueAt(sound,index,value):
    if not sound.__class__ == Sound:
        print "setSampleValueAt(sound,index,value): Input is not a sound."
        raise ValueError 
    sound.setSampleValue(index-1,int(value))

def getSampleValueAt(sound,index):
    if not sound.__class__ == Sound:
        print "getSampleValueAt(sound,index): Input is not a sound."
        raise ValueError 
    return sound.getSampleValue(index-1)

def getSampleObjectAt(sound,index):
    if not sound.__class__ == Sound:
        print "getSampleObjectAt(sound,index): Input is not a sound."
        raise ValueError 
    # return sound.getSampleObjectAt(index-1)
    return Sample(sound, index-1)

def setSample(sample,value):
    if not sample.__class__ == Sample:
        print "setSample(sample,value): Input is not a sample."
        raise ValueError 
    if value > 32767:
        value = 32767
    elif value < -32768:
        value = -32768
    # Need to coerce value to integer
    return sample.setValue( int(value) )

def getSample(sample):
    if not sample.__class__ == Sample:
        print "getSample(sample): Input is not a sample."
        raise ValueError 
    return sample.getValue()

def getSound(sample):
    if not sample.__class__ == Sample:
        print "getSound(sample): Input is not a sample."
        raise ValueError 
    return sample.getSound()

def getLength(sound):
    if not sound.__class__ == Sound:
        print "getLength(sound): Input is not a sound."
        raise ValueError 
    return sound.getLength()

def writeSoundTo(sound,filename):
    global mediaFolder
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not sound.__class__ == Sound:
        print "writeSoundTo(sound,filename): Input is not a sound."
        raise ValueError 
    sound.writeToFile(filename)

##
# Globals for styled text
##
def makeStyle(fontName,emph,size):
    return awt.Font(fontName,emph,size)

sansSerif = "SansSerif"
serif = "Serif"
mono = "Monospaced"
italic = awt.Font.ITALIC
bold = awt.Font.BOLD
plain = awt.Font.PLAIN

##
## Global color functions
##


# this class is solely for the purpose of 
# making makeLighter makeDarker work.
# both of these functions destructively modify a color
# and a color in java is a constant value so we have to put
# this python interface here
class Color:
    def __init__(self,r,g=None,b=None):
        if b == None:
            self.color = r
        else:
            self.color = awt.Color(r,g,b)

    def __str__(self):
        return "color r="+str(self.getRed())+" g="+str(self.getGreen())+" b="+str(self.getBlue())

    def __repr__(self):
        return "Color("+str(self.getRed())+", "+str(self.getGreen())+", "+str(self.getBlue())+")"

    def __eq__(self,newcolor):
        return ((self.getRed() == newcolor.getRed()) and (self.getGreen() == newcolor.getGreen()) and (self.getBlue() == newcolor.getBlue()))
   
    def __ne__(self,newcolor):
        return (not self.__eq__(newcolor))

    def __tojava__(self, javaclass):
        if javaclass == awt.Color:
            return self.color
        else:
            return self

    def setRGB(self, r, g, b):
        self.color = awt.Color(r, g, b)

    def getRed(self):
        return self.color.getRed()
   
    def getGreen(self):
        return self.color.getGreen()
   
    def getBlue(self):
        return self.color.getBlue()

    def distance(self, othercolor):
        r = pow((self.getRed() - othercolor.getRed()),2)
        g = pow((self.getGreen() - othercolor.getGreen()),2)
        b = pow((self.getBlue() - othercolor.getBlue()) ,2)
        return math.sqrt(r+g+b)

def pickAColor():
    retValue = swing.JColorChooser().showDialog(swing.JFrame(),"Choose a color", awt.Color(0,0,0))
    if retValue != None:
        return Color(retValue.getRed(),retValue.getGreen(),retValue.getBlue())
    else:
        return Color(0,0,0)

#Constants
black = Color(0,0,0)
white = Color(255,255,255)
blue = Color(0,0,255)
red = Color(255,0,0)
green = Color(0,255,0)
gray = Color(128,128,128)
darkGray = Color(64,64,64)
lightGray = Color(192,192,192)
yellow = Color(255,255,0)
orange = Color(255,200,0)
pink = Color(255,175,175)
magenta = Color(255,0,255)
cyan = Color(0,255,255)

##
## Global picture functions
##
def makePicture(filename):
    global mediaFolder
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not os.path.isfile(filename):
        print "makePicture(filename): There is no file at "+filename
        raise ValueError 
    picture = Picture()
    picture.loadOrFail(filename)
    return picture

# MMO (1 Dec 2005): Capped width/height to max 10000 and min 1
# alexr (6 Sep 2006): fixed to work without the Python classes.
def makeEmptyPicture(width, height):
    if width > 10000 or height > 10000:
        print "makeEmptyPicture(width, height): height and width must be less than 10000 each"
        raise ValueError
    if width <= 0 or height <= 0:
        print "makeEmptyPicture(width, height): height and width must be greater than 0 each"
        raise ValueError
    picture = Picture(width, height)
    # picture.createImage(width, height)
    # picture.filename = ''
    # careful here; do we want empty strings or "None"?
    return picture
    
def getPixels(picture):
    if not picture.__class__ == Picture:
        print "getPixels(picture): Input is not a picture"
        raise ValueError 
    return picture.getPixels()

def getAllPixels(picture):
        return getPixels(picture)

def getWidth(picture):
    if not picture.__class__ == Picture:
        print "getWidth(picture): Input is not a picture"
        raise ValueError 
    return picture.getWidth()

def getHeight(picture):
    if not picture.__class__ == Picture:
        print "getHeight(picture): Input is not a picture"
        raise ValueError 
    return picture.getHeight()

def show(picture, title=None):
    #picture.setTitle(getShortPath(picture.filename))
    #if title <> None:
            #picture.setTitle(title)
    if not picture.__class__ == Picture:
        print "show(picture): Input is not a picture"
        raise ValueError 
    picture.show()

def repaint(picture):
    if not (picture.__class__ == World or picture.__class__ == Picture):
        print "repaint(picture): Input is not a picture or a world"
        raise ValueError 
    picture.repaint()

## adding graphics to your pictures! ##
def addLine(picture, x1, y1, x2, y2, acolor=black):
    if not picture.__class__ == Picture:
        print "addLine(picture,acolor,x1,y1,x2,y2): Input is not a picture"
        raise ValueError 
    g = picture.getBufferedImage().createGraphics()
    g.setColor(acolor.color)
    g.drawLine(x1 - 1,y1 - 1,x2 - 1,y2 - 1)
    # picture.addLine(black,x1,y1,x2,y2)

def addText(picture, x, y, string, acolor=black):
    if not picture.__class__ == Picture:
        print "addText(picture,acolor,x,y,string): Input is not a picture"
        raise ValueError 
    g = picture.getBufferedImage().getGraphics()
    g.setColor(acolor.color)
    g.drawString(string, x - 1, y - 1)

def addRect(picture, x,y,w,h, acolor=black):
    if not picture.__class__ == Picture:
        print "addRect(picture,acolor,x,y,w,h): Input is not a picture"
        raise ValueError 
    g = picture.getBufferedImage().getGraphics()
    g.setColor(acolor.color)
    g.drawRect(x - 1,y - 1,w,h)

def addRectFilled(picture,x,y,w,h, acolor=black):
    if not picture.__class__ == Picture:
        print "addRectFilled(picture,acolor,x,y,w,h,acolor): Input is not a picture"
        raise ValueError 
    g = picture.getBufferedImage().getGraphics()
    g.setColor(acolor.color)
    g.fillRect(x - 1,y - 1,w,h)


# this needs to be set to the same value as self.wrapPixelValues in JESUI.py
wrapAroundPixelValues = 0

def checkPixel(raw):
        """Assuming that the value is 0-255"""
        value = int(raw)

        if value < 0 or value > 255:
                if wrapAroundPixelValues:
                        return (value % 256)

        return value

## note the -1; in JES we think of pictures as starting at (1,1) but not
## in the Java.
def getPixel(picture,x,y):
    if not picture.__class__ == Picture:
        print "getPixel(picture,x,y): Input is not a picture"
        raise ValueError 
    if x < 1 or x > getWidth(picture):
        print "getPixel(picture,x,y): x (= %s) is less than 1 or bigger than the width (= %s)" % (x,getWidth(picture))
        raise ValueError
    if y < 1 or y > getHeight(picture):
        print "getPixel(picture,x,y): y (= %s) is less than 1 or bigger than the height (= %s)" % (y,getHeight(picture))
        raise ValueError

    return picture.getPixel(x - 1, y - 1)
    
def setRed(pixel,value):
    value = checkPixel(value)
    if not pixel.__class__ == Pixel:
        print "setRed(pixel,value): Input is not a pixel"
        raise ValueError 
    pixel.setRed(value)

def getRed(pixel):
    if not pixel.__class__ == Pixel:
        print "getRed(pixel): Input is not a pixel"
        raise ValueError 
    return pixel.getRed()

def setBlue(pixel,value):
    value = checkPixel(value)
    if not pixel.__class__ == Pixel:
        print "setBlue(pixel,value): Input is not a pixel"
        raise ValueError 
    pixel.setBlue(value)

def getBlue(pixel):
    if not pixel.__class__ == Pixel:
        print "getBlue(pixel): Input is not a pixel"
        raise ValueError 
    return pixel.getBlue()

def setGreen(pixel,value):
    value = checkPixel(value)
    if not pixel.__class__ == Pixel:
        print "setGreen(pixel,value): Input is not a pixel"
        raise ValueError 
    pixel.setGreen(value)

def getGreen(pixel):
    if not pixel.__class__ == Pixel:
        print "getGreen(pixel): Input is not a pixel"
        raise ValueError 
    return pixel.getGreen()

def getColor(pixel):
    if not pixel.__class__ == Pixel:
        print "getColor(pixel): Input is not a pixel"
        raise ValueError 
    return Color(pixel.getColor())

def setColor(pixel,color):
    if not pixel.__class__ == Pixel:
        print "setColor(pixel,color): Input is not a pixel."
        raise ValueError 
    if not color.__class__ == Color:
        print "setColor(pixel,color): Input is not a color."
        raise ValueError 
    pixel.setColor(color.color)
    
def getX(pixel):
    if not pixel.__class__ == Pixel:
        print "getX(pixel): Input is not a pixel"
        raise ValueError 
    return pixel.getX() + 1

def getY(pixel):
    if not pixel.__class__ == Pixel:
        print "getY(pixel): Input is not a pixel"
        raise ValueError 
    return pixel.getY() + 1

def distance(c1,c2):
    if not c1.__class__ == Color:
        print "distance(c1,c2): First input is not a color."
        raise ValueError 
    if not c2.__class__ == Color:
        print "distance(c1,c2): Second input is not a color."
        raise ValueError 
    return c1.distance(c2)

def writePictureTo(pict,filename):
    global mediaFolder
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not pict.__class__ == Picture:
        print "writePictureTo(pict,filename): Input is not a picture"
        raise ValueError
    pict.writeOrFail(filename)
#   if not os.path.exists(filename):
#       print "writePictureTo(pict,filename): Path is not valid"
#       raise ValueError



# not to be confused with setColor, totally different, don't document/export
def setColorTo(color, other):
    color.setRGB(other.getRed(), other.getGreen(), other.getBlue())
    return color

def makeDarker(color):
    """This function has side effects on purpose, see p49 """
    return setColorTo(color, color.color.darker())

def makeLighter(color):
    """This function has side effects on purpose"""
    return setColorTo(color, color.color.brighter())

def makeBrighter(color):
    """This function has side effects on purpose"""
    return setColorTo(color, color.color.brighter())

def makeColor(red,green,blue):
    return Color(int(red),int(green),int(blue))

def setAllPixelsToAColor(picture,color):
    """This function sets the picture to one color"""
    if not picture.__class__ == Picture:
        print "setAllPixelsToAColor: First argument is not a picture"
        raise ValueError 
    if not color.__class__ == Color:
        print "setAllPixelsToAColor: Second argument is not a color"
        raise ValueError 
    picture.setAllPixelsToAColor(color.color)


def copyInto(smallPicture, bigPicture, startX, startY):
    """like copyInto(butterfly, jungle, 20,20)"""
    if not smallPicture.__class__ == Picture:
        print "copyInto(smallPicture, bigPicture, startX, startY): smallPicture must be a picture"
        raise ValueError 
    if not bigPicture.__class__ == Picture:
        print "copyInto(smallPicture, bigPicture, startX, startY): bigPicture must be a picture"
        raise ValueError 
    if startX < 1 or startX > getWidth(bigPicture):
        print "copyInto(smallPicture, bigPicture, startX, startY): startX must be within the bigPicture"
        raise ValueError
    if startY < 1 or startY > getHeight(bigPicture):
        print "copyInto(smallPicture, bigPicture, startX, startY): startY must be within the bigPicture"
        raise ValueError
    if startX + getWidth(smallPicture) > getWidth(bigPicture) or \
            startY + getHeight(smallPicture) > getHeight(bigPicture):
        print "copyInto(smallPicture, bigPicture, startX, startY): smallPicture won't fit into bigPicture"
        raise ValueError

    xOffset = startX - 1
    yOffset = startY - 1

    for x in range(0, getWidth(smallPicture)):
        for y in range(0, getHeight(smallPicture)):
            bigPicture.setBasicPixel(x + xOffset, y + yOffset, smallPicture.getBasicPixel(x,y))

    return bigPicture


def duplicatePicture(picture):
    """returns a copy of the picture"""
    if not picture.__class__ == Picture:
        print "duplicatePicture: First argument is not a picture"
        raise ValueError 
    return Picture(picture)


##
# Input and Output interfaces
##

#  radius = SimpleInput.getNumber("Enter the radius of the cylinder")
#  SimpleOutput.showInformation("The volume of the cylinder is %.02f " % volume)

def requestNumber(message):
    return SimpleInput.getNumber(message)

def requestInteger(message):
    return SimpleInput.getIntNumber(message)

def requestIntegerInRange(message, min, max):
    if min >= max:
        print "requestIntegerInRange(message, min, max): min >= max not allowed"
        raise ValueError

    return SimpleInput.getIntNumber(message, min, max)

def requestString(message):
    return SimpleInput.getString(message)

def showWarning(message):
    return SimpleOutput.showWarning(message)

def showInformation(message):
    return SimpleOutput.showInformation(message)

def showError(message):
    return SimpleOutput.showError(message)


##
# Java Music Interface
##
def playNote(note, duration, intensity=64):
    JavaMusic.playNote(note, duration, intensity)


##
# General user tools
#

def pickAFile():
    global _lastFilePath
    import javax.swing
    if _lastFilePath != None and os.path.exists(_lastFilePath):
        choose = javax.swing.JFileChooser(_lastFilePath)
    else:
        choose = javax.swing.JFileChooser()
    choose.setDialogTitle("Pick A File")
    jf = javax.swing.JFrame()
    jf.getContentPane().add(choose)
    retValue = choose.showOpenDialog(jf)
    if retValue == 0:
        path = choose.getSelectedFile().getAbsolutePath()
        _lastFilePath = os.path.dirname(path)
        return path
    else:
        return ""
                
def pickAFolder():
    global _lastFilePath
    import javax.swing
    if _lastFilePath != None and os.path.exists(_lastFilePath):
        choose = javax.swing.JFileChooser(_lastFilePath)
    else:
        choose = javax.swing.JFileChooser()
    choose.setDialogTitle("Pick A Folder")
    choose.setFileSelectionMode(javax.swing.JFileChooser.DIRECTORIES_ONLY)
    choose.setFileHidingEnabled(1)
    jf = javax.swing.JFrame()
    jf.getContentPane().add(choose)
    retValue = choose.showOpenDialog(jf)
    if retValue == 0:
        path = choose.getSelectedFile().getAbsolutePath()
        _lastFilePath = path
        return path
    else:
        return ""

def quit():
    sys.exit(0)
        
##
# MediaTools interface
#

def openPictureTool(picture):
    import PictureExplorer
    thecopy = duplicatePicture(picture)
    viewer = PictureExplorer(thecopy)

    viewer.changeToBaseOne();
    viewer.setTitle(getShortPath(picture.getFileName() ))
    
def openFrameSequencerTool(movie):
    FrameSequencerTool.FrameSequencerTool(movie)

def openSoundTool(sound):
    import SoundExplorer
    thecopy = Sound(sound)
    viewer = SoundExplorer(thecopy, 0)
    try:
        viewer.setTitle(getShortPath(sound.getFileName()))
    except:
        viewer.setTitle("No File Name")

# let's try the turtles...
import Turtle
import World

def turn(turtle, degrees=90):
    if not turtle.__class__ == Turtle:
        print "turn(turtle, degrees): Input is not a turtle"
        raise ValueError 
    else:
        turtle.turn(degrees)

def turnRight(turtle):
    if not turtle.__class__ == Turtle:
        print "turnRight(turtle, degrees): Input is not a turtle"
        raise ValueError 
    else:
        turtle.turnRight()

def turnToFace(turtle, x, y):
    if not turtle.__class__ == Turtle:
        print "turnToFace(turtle, x, y): Input is not a turtle"
        raise ValueError 
    else:
        turtle.turnToFace(x, y)

def turnLeft(turtle):
    if not turtle.__class__ == Turtle:
        print "turnLeft(turtle, degrees): Input is not a turtle"
        raise ValueError 
    else:
        turtle.turnLeft()

def forward(turtle, pixels=100):
    if not turtle.__class__ == Turtle:
        print "forward(turtle, pixels): Input is not a turtle"
        raise ValueError 
    else:
        turtle.forward(pixels)

def backward(turtle, pixels=100):
    if not turtle.__class__ == Turtle:
        print "forward(turtle, pixels): Input is not a turtle"
        raise ValueError 
    if (None == pixels):
        turtle.backward()
    else:
        turtle.backward(pixels)

def moveTo(turtle, x, y):
    if not turtle.__class__ == Turtle:
        print "forward(turtle, pixels): Input is not a turtle"
        raise ValueError 
    turtle.moveTo(x,y)

def makeTurtle(world):
    if not (world.__class__ == World or world.__class__ == Picture):
        print "makeTurtle(world): Input is not a world or picture"
        raise ValueError 
    turtle = Turtle(world)
    return turtle

def penUp(turtle):
    if not turtle.__class__ == Turtle:
        print "penUp(turtle): Input is not a turtle"
        raise ValueError 
    turtle.penUp()

def penDown(turtle):
    if not turtle.__class__ == Turtle:
        print "penDown(turtle): Input is not a turtle"
        raise ValueError 
    turtle.penDown()

def drop(turtle, picture):
    if not turtle.__class__ == Turtle:
        print "drop(turtle, picture): Input is not a turtle"
        raise ValueError 
    if not picture.__class__ == Picture:
        print "drop(turtle, picture): Input is not a picture"
        raise ValueError 
    turtle.drop(picture)

def getXPos(turtle):
    if not turtle.__class__ == Turtle:
        print "drop(turtle, picture): Input is not a turtle"
        raise ValueError 
    return turtle.getXPos()

def getYPos(turtle):
    if not turtle.__class__ == Turtle:
        print "drop(turtle, picture): Input is not a turtle"
        raise ValueError 
    return turtle.getYPos()

## add these things: turnToFace(turtle, another turtle)
## getHeading, getXPos, getYPos

## world methods
def makeWorld(width=None, height=None):
    if(width and height):
        w = World(width, height)
    else:
        w = World()
    return w

def getTurtleList(world):
    if not world.__class__ == World:
        print "getTurtleList(world): Input is not a world"
        raise ValueError 
    return world.getTurtleList()

# end of stuff imported for worlds and turtles

# used in the book
def printNow(text):
    sys.stdout.printNow("%s\n" % text)

class Movie:
    def __init__(self): # frames are filenames
        self.frames = []
    
    def addFrame(self, frame):
        self.frames.append(frame)
    
    def __len__(self):
        return len(self.frames)

    def __str__(self):
        return "Movie, frames: "+str(len(self))
    
    def __repr__(self):
        return "Movie, frames: "+str(len(self))
    
    def __getitem__(self,item):
        return self.frames[item]

    def writeFramesToDirectory(self, directory):
        import FrameSequencer
        fs = FrameSequencer(directory)
        for frameindex in range(0, self.listModel.size()):
            fs.addFrame(Picture(self.listModel.get(frameindex)))
            #fs.play(self.fps)

    def play(self):
        import java.util.ArrayList as ArrayList
        import MoviePlayer
        list = ArrayList()
        for f in self.frames:
            list.add(f)
        MoviePlayer(list)

def makeMovie():
    return Movie()

def makeMovieFromInitialFile(filename):
    import re
    movie = Movie()

    sep_location = filename.rfind(os.sep)
    if(-1 == sep_location):
        filename = mediaFolder + filename

    movie.directory = filename[:(filename.rfind(os.sep))]
    movie.init_file = filename[(filename.rfind(os.sep))+1:]
    regex = re.compile('[0-9]+')
    file_regex = regex.sub('.*', movie.init_file)
    
    for item in os.listdir(movie.directory):
        if re.match(file_regex, item):
            movie.addFrame(movie.directory + os.sep + item)
    
    return movie

def addFrameToMovie(a, b):
    frame = None
    movie = None
    if a.__class__ == Movie:
        movie = a
        frame = b
    else:
        movie = b
        frame = a

    if movie.__class__ != Movie or frame.__class__ != String:
        print "addFrameToMovie(frame, movie): frame is not a string or movie is not a Movie object."
        raise ValueError
    
    movie.addFrame(frame)


def writeFramesToDirectory(movie, directory=None):
    if movie.__class__ != Movie:
        print "writeFramesToDirectory(movie, directory): movie is not a Movie object."
        raise ValueError

    if directory == None:
        directory = user.home

    movie.writeFramesToDirectory(directory)

def playMovie(movie):
    if movie.__class__ != Movie:
        print "playMovie(movie): movie is not a Movie object."
        raise ValueError
    movie.play()
